@@ -1,6 +1,15 @@
 from vfastpunct.constants import LOGGER
 
 import requests
+import hashlib
+
+
+def md5(fname):
+    hash_md5 = hashlib.md5()
+    with open(fname, "rb") as f:
+        for chunk in iter(lambda: f.read(4096), b""):
+            hash_md5.update(chunk)
+    return hash_md5.hexdigest()
 
 
 def download_file_from_google_drive(id, destination, confirm=None):
